<?php

/**
 * @copyright	Copyright (C) 2011 1sr, Inc. All rights reserved.
 * @author		1sr
 * @link		http://1sr.de/fsramazon
 * @license		http://www.gnu.org/copyleft/gpl.html GNU/GPL
 */

require_once 'resultset.php';

class amazon
{
	/**
     * amazon web services access key
     *
     * @var string
     */
    protected $_awsAccessKeyId;

    /**
     * @var string
     */ 
	protected $_awsSecretAccessKeyId = null;
	
	/**
     * @var string
     */ 
	protected $_associateTag = null;

    /**
     * @var string
     */
    protected $_baseUri = null;
    
    /**
     * list of amazon web service base URLs, indexed by country code
     *
     * @var array
     */
	protected $_baseUriList = array('ca' => 'http://ecs.amazonaws.ca',
									'cn' => 'http://webservices.amazon.cn',
							        'de' => 'http://ecs.amazonaws.de',
							        'fr' => 'http://ecs.amazonaws.fr',
							        'it' => 'http://webservices.amazon.it',							         
							        'jp' => 'http://ecs.amazonaws.jp',
									'es' => 'http://webservices.amazon.es',
									'uk' => 'http://ecs.amazonaws.co.uk',
									'us' => 'http://ecs.amazonaws.com');
							        
	 /**
	  * use of `file_get_contents` may not work on all servers because it relies on the ability to
	  * open remote URLs using the file manipulation functions. PHP gives you the ability to disable 
	  * this functionality in your php.ini file and many administrators do so for security reasons.
	  * 
	  * If your administrator has not done so, `file_get_contents` will be used (default mode).
	  * 
	  * @var boolean  
	  */
	 protected $_fileGetContentsMode;
	 
	 /**
	  * creates an instance of amazon
	  * 
	  * @param  string $awsAccessKeyId
	  * @param  string $countryCode
	  * @param  string $associateTag
	  * @param  string $awsSecretAccessKeyId
	  * @param  boolean $fileGetContentsMode
	  * @return void
	  */
	 public function __construct($awsAccessKeyId, $countryCode = 'us', $associateTag = null, $awsSecretAccessKeyId = null, $fileGetContentsMode = true)
	 {
	 	$this->_fileGetContentsMode = $fileGetContentsMode;
    	
    	$this->_awsAccessKeyId = (string) $awsAccessKeyId;
    	$this->_awsSecretAccessKeyId = $awsSecretAccessKeyId;
    	
    	$this->_associateTag = $associateTag;
    	
    	$countryCode = (string) $countryCode;
        if (!isset($this->_baseUriList[$countryCode]))
        {
            throw new Exception("Unknown country code: $countryCode");
        }

        $this->_baseUri = $this->_baseUriList[$countryCode];  	
	 }
	 
	 /**
	  * builds the amazon request
	  */
	 private function buildRequest($options)
	 {
		// some paramters
	    $method = "GET";
	    $host = parse_url($this->_baseUri, PHP_URL_HOST);
	    $path = "/onca/xml";
	 
	    // additional parameters
	    $options["Service"] = "AWSECommerceService";
	    $options["AWSAccessKeyId"] = $this->_awsAccessKeyId;
	    $options["AssociateTag"] = $this->_associateTag;
	    // set GMT timestamp
	    $options["Timestamp"] = gmdate("Y-m-d\TH:i:s\Z");
	    // version of amazon API
	    $options["Version"] = "2011-08-01";
	 
	    // sort the options
	    ksort($options);

		$requestParams = array();
		
		foreach(array_keys($options) as $key)
		{
			$requestParams[] = rawurlencode($key).'='.rawurlencode($options[$key]);
		}	
		
		$requestParams = implode("&", $requestParams);
	 
	    // create the sign string
	    $signStr = $method."\n".$host."\n".$path."\n".$requestParams;
	 
	    // calculate HMAC with SHA256 and base64-encoding
	    $signature = base64_encode(hash_hmac("sha256", $signStr, $this->_awsSecretAccessKeyId, True));
	 
	    // encode the signature for the request
	    $signature = rawurlencode($signature);
	 
	    // create request
	    $request = "http://".$host.$path."?".$requestParams."&Signature=".$signature;
	 	
	    // request data	    
	    if($this->_fileGetContentsMode)
	    {
	    	$response = file_get_contents($request);
	    }
	    else
	    {
	    	$session = curl_init($request);
			curl_setopt($session, CURLOPT_HEADER, false);
			curl_setopt($session, CURLOPT_RETURNTRANSFER, true);
			$response = curl_exec($session);
			curl_close($session);
	    }
		
	 	
	    if (!$response)
	    {
	        return false;
	    }
	    else
	    {	    	
	        // parse response xml string
	        $pxml = simplexml_load_string($response);
	               	
			return $pxml;
	    }
	 }
	
	 /**
	 * returns items that satisfy the search criteria, including one or more search indices
	 */
	 public function itemSearch($options)
	 {

		$options['Operation'] = 'ItemSearch';

		if(!isset($options['ItemPage']))
		{
			$options['ItemPage'] = 1;
		}
		
		if(!isset($options['ResponseGroup']))
		{
			$options['ResponseGroup'] = 'Large,VariationSummary';
		}
		
		$pxml = $this->buildRequest($options);
		
		if (!$pxml)
		{
		    throw new Exception("An error occured searching for products.");
		}
		else
		{
			return new resultset($pxml);
	 	}
	 }
	 
	 public function itemLookup($options, $throwException = true)
	 {

		$options['Operation'] = 'ItemLookup';
		
		if(!isset($options['IdType']))
		{
			$options['IdType'] = 'ASIN';
		}
		
		if(!isset($options['ResponseGroup']))
		{
			$options['ResponseGroup'] = 'Large,Similarities,Reviews,VariationSummary';
			$options['TruncateReviewsAt'] = '256';
		}
		
		$pxml = $this->buildRequest($options);
		
		if (!$pxml && $throwException)
		{
		    throw new Exception("An error occured lookup a product.");
		}
		else
		{
			return $pxml->Items->Item[0];
	 	}
	 }

}
